@@ -1,24 +1,37 @@
 import json
+from typing import Dict, Iterable, List, Tuple, Union
 
 from dateutil.parser import parse as dtparse
 
 from moto.core.responses import BaseResponse
-from moto.core.utils import amzn_request_id
-from .models import cloudwatch_backends, MetricDataQuery, MetricStat, Metric, Dimension
-from .exceptions import InvalidParameterCombination
+
+from .exceptions import InvalidParameterCombination, ValidationError
+from .models import (
+    CloudWatchBackend,
+    Dimension,
+    FakeAlarm,
+    Metric,
+    MetricDataQuery,
+    MetricStat,
+    cloudwatch_backends,
+)
+
+ERROR_RESPONSE = Tuple[str, Dict[str, int]]
 
 
 class CloudWatchResponse(BaseResponse):
-    @property
-    def cloudwatch_backend(self):
-        return cloudwatch_backends[self.region]
+    def __init__(self) -> None:
+        super().__init__(service_name="cloudwatch")
 
-    def _error(self, code, message, status=400):
+    @property
+    def cloudwatch_backend(self) -> CloudWatchBackend:
+        return cloudwatch_backends[self.current_account][self.region]
+
+    def _error(self, code: str, message: str, status: int = 400) -> ERROR_RESPONSE:
         template = self.response_template(ERROR_RESPONSE_TEMPLATE)
         return template.render(code=code, message=message), dict(status=status)
 
-    @amzn_request_id
-    def put_metric_alarm(self):
+    def put_metric_alarm(self) -> str:
         name = self._get_param("AlarmName")
         namespace = self._get_param("Namespace")
         metric_name = self._get_param("MetricName")
@@ -27,14 +40,14 @@ class CloudWatchResponse(BaseResponse):
         if metrics:
             metric_data_queries = []
             for metric in metrics:
-                dimensions = []
+                metric_dimensions = []
                 dims = (
                     metric.get("MetricStat", {})
                     .get("Metric", {})
                     .get("Dimensions.member", [])
                 )
                 for dim in dims:
-                    dimensions.append(
+                    metric_dimensions.append(
                         Dimension(name=dim.get("Name"), value=dim.get("Value"))
                     )
                 metric_stat = None
@@ -48,7 +61,7 @@ class CloudWatchResponse(BaseResponse):
                         metric=Metric(
                             metric_name=stat_metric_name,
                             namespace=stat_metric_ns,
-                            dimensions=dimensions,
+                            dimensions=metric_dimensions,
                         ),
                         period=stat_details.get("Period"),
                         stat=stat_details.get("Stat"),
@@ -76,7 +89,7 @@ class CloudWatchResponse(BaseResponse):
         dimensions = self._get_list_prefix("Dimensions.member")
         alarm_actions = self._get_multi_param("AlarmActions.member")
         ok_actions = self._get_multi_param("OKActions.member")
-        actions_enabled = self._get_param("ActionsEnabled")
+        actions_enabled = self._get_bool_param("ActionsEnabled")
         insufficient_data_actions = self._get_multi_param(
             "InsufficientDataActions.member"
         )
@@ -117,8 +130,7 @@ class CloudWatchResponse(BaseResponse):
         template = self.response_template(PUT_METRIC_ALARM_TEMPLATE)
         return template.render(alarm=alarm)
 
-    @amzn_request_id
-    def describe_alarms(self):
+    def describe_alarms(self) -> str:
         action_prefix = self._get_param("ActionPrefix")
         alarm_name_prefix = self._get_param("AlarmNamePrefix")
         alarm_names = self._get_multi_param("AlarmNames.member")
@@ -135,7 +147,7 @@ class CloudWatchResponse(BaseResponse):
         elif state_value:
             alarms = self.cloudwatch_backend.get_alarms_by_state_value(state_value)
         else:
-            alarms = self.cloudwatch_backend.get_all_alarms()
+            alarms = self.cloudwatch_backend.describe_alarms()
 
         metric_alarms = [a for a in alarms if a.rule is None]
         composite_alarms = [a for a in alarms if a.rule is not None]
@@ -145,28 +157,32 @@ class CloudWatchResponse(BaseResponse):
             metric_alarms=metric_alarms, composite_alarms=composite_alarms
         )
 
-    @amzn_request_id
-    def delete_alarms(self):
+    def delete_alarms(self) -> str:
         alarm_names = self._get_multi_param("AlarmNames.member")
         self.cloudwatch_backend.delete_alarms(alarm_names)
         template = self.response_template(DELETE_METRIC_ALARMS_TEMPLATE)
         return template.render()
 
-    @amzn_request_id
-    def put_metric_data(self):
+    def put_metric_data(self) -> str:
         namespace = self._get_param("Namespace")
         metric_data = self._get_multi_param("MetricData.member")
         self.cloudwatch_backend.put_metric_data(namespace, metric_data)
         template = self.response_template(PUT_METRIC_DATA_TEMPLATE)
         return template.render()
 
-    @amzn_request_id
-    def get_metric_data(self):
-        start = dtparse(self._get_param("StartTime"))
-        end = dtparse(self._get_param("EndTime"))
-        scan_by = self._get_param("ScanBy")
+    def get_metric_data(self) -> str:
+        params = self._get_params()
+        start = dtparse(params["StartTime"])
+        end = dtparse(params["EndTime"])
+        scan_by = params.get("ScanBy") or "TimestampDescending"
 
-        queries = self._get_list_prefix("MetricDataQueries.member")
+        queries = params.get("MetricDataQueries", [])
+        for query in queries:
+            if "MetricStat" not in query and "Expression" not in query:
+                # AWS also returns the empty line
+                raise ValidationError(
+                    "The parameter MetricDataQueries.member.1.MetricStat is required.\n"
+                )
         results = self.cloudwatch_backend.get_metric_data(
             start_time=start, end_time=end, queries=queries, scan_by=scan_by
         )
@@ -174,8 +190,7 @@ class CloudWatchResponse(BaseResponse):
         template = self.response_template(GET_METRIC_DATA_TEMPLATE)
         return template.render(results=results)
 
-    @amzn_request_id
-    def get_metric_statistics(self):
+    def get_metric_statistics(self) -> str:
         namespace = self._get_param("Namespace")
         metric_name = self._get_param("MetricName")
         start_time = dtparse(self._get_param("StartTime"))
@@ -206,8 +221,7 @@ class CloudWatchResponse(BaseResponse):
         template = self.response_template(GET_METRIC_STATISTICS_TEMPLATE)
         return template.render(label=metric_name, datapoints=datapoints)
 
-    @amzn_request_id
-    def list_metrics(self):
+    def list_metrics(self) -> str:
         namespace = self._get_param("Namespace")
         metric_name = self._get_param("MetricName")
         dimensions = self._get_params().get("Dimensions", [])
@@ -218,25 +232,22 @@ class CloudWatchResponse(BaseResponse):
         template = self.response_template(LIST_METRICS_TEMPLATE)
         return template.render(metrics=metrics, next_token=next_token)
 
-    @amzn_request_id
-    def delete_dashboards(self):
+    def delete_dashboards(self) -> Union[str, ERROR_RESPONSE]:
         dashboards = self._get_multi_param("DashboardNames.member")
-        if dashboards is None:
+        if not dashboards:
             return self._error("InvalidParameterValue", "Need at least 1 dashboard")
 
-        status, error = self.cloudwatch_backend.delete_dashboards(dashboards)
-        if not status:
+        error = self.cloudwatch_backend.delete_dashboards(dashboards)
+        if error is not None:
             return self._error("ResourceNotFound", error)
 
         template = self.response_template(DELETE_DASHBOARD_TEMPLATE)
         return template.render()
 
-    @amzn_request_id
-    def describe_alarm_history(self):
-        raise NotImplementedError()
-
     @staticmethod
-    def filter_alarms(alarms, metric_name, namespace):
+    def filter_alarms(
+        alarms: Iterable[FakeAlarm], metric_name: str, namespace: str
+    ) -> List[FakeAlarm]:
         metric_filtered_alarms = []
 
         for alarm in alarms:
@@ -244,25 +255,21 @@ class CloudWatchResponse(BaseResponse):
                 metric_filtered_alarms.append(alarm)
         return metric_filtered_alarms
 
-    @amzn_request_id
-    def describe_alarms_for_metric(self):
-        alarms = self.cloudwatch_backend.get_all_alarms()
+    def describe_alarms_for_metric(self) -> str:
+        alarms = self.cloudwatch_backend.describe_alarms()
         namespace = self._get_param("Namespace")
         metric_name = self._get_param("MetricName")
         filtered_alarms = self.filter_alarms(alarms, metric_name, namespace)
         template = self.response_template(DESCRIBE_METRIC_ALARMS_TEMPLATE)
         return template.render(alarms=filtered_alarms)
 
-    @amzn_request_id
-    def disable_alarm_actions(self):
+    def disable_alarm_actions(self) -> str:
         raise NotImplementedError()
 
-    @amzn_request_id
-    def enable_alarm_actions(self):
+    def enable_alarm_actions(self) -> str:
         raise NotImplementedError()
 
-    @amzn_request_id
-    def get_dashboard(self):
+    def get_dashboard(self) -> Union[str, ERROR_RESPONSE]:
         dashboard_name = self._get_param("DashboardName")
 
         dashboard = self.cloudwatch_backend.get_dashboard(dashboard_name)
@@ -272,8 +279,7 @@ class CloudWatchResponse(BaseResponse):
         template = self.response_template(GET_DASHBOARD_TEMPLATE)
         return template.render(dashboard=dashboard)
 
-    @amzn_request_id
-    def list_dashboards(self):
+    def list_dashboards(self) -> str:
         prefix = self._get_param("DashboardNamePrefix", "")
 
         dashboards = self.cloudwatch_backend.list_dashboards(prefix)
@@ -281,8 +287,7 @@ class CloudWatchResponse(BaseResponse):
         template = self.response_template(LIST_DASHBOARD_RESPONSE)
         return template.render(dashboards=dashboards)
 
-    @amzn_request_id
-    def put_dashboard(self):
+    def put_dashboard(self) -> Union[str, ERROR_RESPONSE]:
         name = self._get_param("DashboardName")
         body = self._get_param("DashboardBody")
 
@@ -296,8 +301,7 @@ class CloudWatchResponse(BaseResponse):
         template = self.response_template(PUT_DASHBOARD_RESPONSE)
         return template.render()
 
-    @amzn_request_id
-    def set_alarm_state(self):
+    def set_alarm_state(self) -> str:
         alarm_name = self._get_param("AlarmName")
         reason = self._get_param("StateReason")
         reason_data = self._get_param("StateReasonData")
@@ -310,8 +314,7 @@ class CloudWatchResponse(BaseResponse):
         template = self.response_template(SET_ALARM_STATE_TEMPLATE)
         return template.render()
 
-    @amzn_request_id
-    def list_tags_for_resource(self):
+    def list_tags_for_resource(self) -> str:
         resource_arn = self._get_param("ResourceARN")
 
         tags = self.cloudwatch_backend.list_tags_for_resource(resource_arn)
@@ -319,8 +322,7 @@ class CloudWatchResponse(BaseResponse):
         template = self.response_template(LIST_TAGS_FOR_RESOURCE_TEMPLATE)
         return template.render(tags=tags)
 
-    @amzn_request_id
-    def tag_resource(self):
+    def tag_resource(self) -> str:
         resource_arn = self._get_param("ResourceARN")
         tags = self._get_multi_param("Tags.member")
 
@@ -329,8 +331,7 @@ class CloudWatchResponse(BaseResponse):
         template = self.response_template(TAG_RESOURCE_TEMPLATE)
         return template.render()
 
-    @amzn_request_id
-    def untag_resource(self):
+    def untag_resource(self) -> str:
         resource_arn = self._get_param("ResourceARN")
         tag_keys = self._get_multi_param("TagKeys.member")
 
@@ -354,7 +355,7 @@ DESCRIBE_ALARMS_TEMPLATE = """<DescribeAlarmsResponse xmlns="http://monitoring.a
         <{{tag_name}}>
             {% for alarm in alarms %}
             <member>
-                <ActionsEnabled>{{ alarm.actions_enabled }}</ActionsEnabled>
+                <ActionsEnabled>{{ "true" if alarm.actions_enabled else "false" }}</ActionsEnabled>
                 <AlarmActions>
                     {% for action in alarm.alarm_actions %}
                     <member>{{ action }}</member>
@@ -480,7 +481,7 @@ DESCRIBE_METRIC_ALARMS_TEMPLATE = """<DescribeAlarmsForMetricResponse xmlns="htt
         <MetricAlarms>
             {% for alarm in alarms %}
             <member>
-                <ActionsEnabled>{{ alarm.actions_enabled }}</ActionsEnabled>
+                <ActionsEnabled>{{ "true" if alarm.actions_enabled else "false" }}</ActionsEnabled>
                 <AlarmActions>
                     {% for action in alarm.alarm_actions %}
                     <member>{{ action }}</member>
